;; This is a nice example of hygienic macro expansion
;; from Michael Adams' paper "Towards the Essence of
;; Hygiene" (POPL 2015). The example shows that renaming
;; bound variables and symbols introduced by macros
;; is insufficient to get full hygiene.

(define x 3)

;; (let-inc u v) evaluates v in a context where u has its outer
;; value plus one. The idea is that this macro requires both
;; knowledge about what u is bound to, and what u might bind
;; in v.
(define-syntax let-inc
  (syntax-rules ()
    ((let-inc u v)
     (let ((u (exact+ 1 u)))
       v))))

(define-syntax mac
  (syntax-rules ()
    ((mac y)
     ;; If the 'x's here were freshly generated, they
     ;; could not refer to the outer 'x' anymore. If they
     ;; were kept as-is, they could inadvertently capture any
     ;; 'x's that might occur in 'y'.
     (let-inc x (exact* x y)))))

(test-group
  (eqv? 12 (mac x)))
